#include "hexutils.h"

int
ParseHexRecord( hexRecord * record, char *inputRecord, int inputRecordLen )
{
  int           index = 0;

  if ( ( record == NULL ) || ( inputRecord == NULL ) )
  {
    return HEX_REC_NULL_PTR;
  }

  if ( inputRecord[0] != HEX_REC_START_CODE )
  {
    return HEX_REC_INVALID_FORMAT;
  }

  record->rec_data_len =
      ConvertHexASCIIByteToInt( inputRecord[1], inputRecord[2] );
  record->rec_address =
      ( ( unsigned short ) ConvertHexASCIIByteToInt( inputRecord[3],
      inputRecord[4] ) << 8 ) +
      ( ( unsigned short ) ConvertHexASCIIByteToInt( inputRecord[5],
      inputRecord[6] ) );
  record->rec_type =
      ConvertHexASCIIByteToInt( inputRecord[7], inputRecord[8] );
  record->rec_checksum =
      ConvertHexASCIIByteToInt( inputRecord[9 + ( record->rec_data_len * 2 )],
      inputRecord[9 + ( record->rec_data_len * 2 ) + 1] );
  record->calc_checksum =
      record->rec_data_len + ( ( record->rec_address >> 8 ) & 0xFF ) +
      ( record->rec_address & 0xFF ) + record->rec_type;

  for ( index = 0; index < record->rec_data_len; index++ )
  {
    record->rec_data[index] =
	ConvertHexASCIIByteToInt( inputRecord[9 + ( index * 2 )],
	inputRecord[9 + ( index * 2 ) + 1] );
    record->calc_checksum += record->rec_data[index];
  }

  record->calc_checksum = ~record->calc_checksum + 1;

  if ( record->calc_checksum != record->rec_checksum )
  {
    return HEX_REC_BAD_CHECKSUM;
  }

  return HEX_REC_OK;
}

int
ConvertHexASCIIDigitToInt( char c )
{
  if ( ( c >= 'a' ) && ( c <= 'f' ) )
    return ( c - 'a' ) + 0x0A;
  else if ( ( c >= 'A' ) && ( c <= 'F' ) )
    return ( c - 'A' ) + 0x0A;
  else if ( ( c >= '0' ) && ( c <= '9' ) )
    return ( c - '0' );
  else
    return -1;
}

int
ConvertHexASCIIByteToInt( char msb, char lsb )
{
  return ( ( ConvertHexASCIIDigitToInt( msb ) << 4 ) +
      ConvertHexASCIIDigitToInt( lsb ) );
}
