#include "lpc214x.h"
#include "spi1.h"
#include "delays.h"
#include "uart0.h"
#include "nrf24lu1.h"
#include "armVic.h"
#include "hexutils.h"

//for the following 9 definitions, see Jim Lynch's ARM tutorial
#define PLOCK 0x400

void          Initialize(  );
void          InitializePLL(  );
void          InitializeIO(  );
void          InitializeInterrupts(  );
void          Feed(  );

void          IRQ_Routine(  ) __attribute__ ( ( interrupt( "IRQ" ) ) );
void          FIQ_Routine(  ) __attribute__ ( ( interrupt( "FIQ" ) ) );
void          SWI_Routine(  ) __attribute__ ( ( interrupt( "SWI" ) ) );
void          UNDEF_Routine(  ) __attribute__ ( ( interrupt( "UNDEF" ) ) );
//end Jim Lynch's definitions

void          ExecuteCommand( unsigned char command );
void          ExecuteCommandProgram(  );
void          ExecuteCommandDumpAllMemory( void );

void          EnableProgrammingPins(  );
void          DisnableProgrammingPins(  );
void          EnablePROGPin(  );
void          DisablePROGPin(  );
void          EnableRSTPin(  );
void          DisableRSTPin(  );
void          SetRSTPin(  );
void          ClearRSTPin(  );
void          SetPROGPin(  );
void          ClearProgPin(  );
void          SetCSNPin(  );
void          ClearCSNPin(  );
void          EnableSPIPins(  );
void          DisableSPIPins(  );

void          put_char( char c );
char          get_char(  );

#define PIN_PROG	0x00200000	// P0.21
#define	PIN_CSN		0x00100000	// P0.20
#define PIN_MOSI	0x00080000	// P0.19
#define PIN_MISO	0x00040000	// P0.18
#define PIN_SCK		0x00020000	// P0.17
#define PIN_RST		0x00008000	// P0.15
#define PIN_RX		0x00000002	// P0.1
#define PIN_TX		0x00000001	// P0.0

#define	PINS_TO_INPUT(io_dir_register, input_mask)	io_dir_register &= ~input_mask
#define	PINS_TO_OUTPUT(io_dir_register, input_mask)	io_dir_register |= input_mask
#define PINS_CLEAR(io_register, input_mask)			io_register &= ~input_mask
#define PINS_SET(io_register, input_mask)			io_register |= input_mask

void          ToggleLED(  );	// toggle the current state of the on-board LED

#define BUF_SIZE 1024
unsigned char tx_buffer[BUF_SIZE];
unsigned int  tx_buf_first = 0;
unsigned int  tx_buf_last = 0;

#define NRF24LU1_FLASH_LOC_START	0x0000
#define NRF24LU1_FLASH_LOC_END		0x3FFF
#define NRF24LU1_FLASH_LEN			((NRF24LU1_FLASH_LOC_END - NRF24LU1_FLASH_LOC_START) + 1)
unsigned char nrf24lu1_flash[NRF24LU1_FLASH_LEN];

void          add_hex_record_to_flash( hexRecord * record );
void          initialize_flash(  );

//main routine
int
main( void )
{
  unsigned char curchar;
  unsigned char command = 0;

  Initialize(  );

  enableIRQ(  );

  uart0_printf( "\r\nPGM> " );

  while ( 1 )
  {
    curchar = get_char(  );

    switch ( curchar )
    {
      case '\r':
	{
	}
	break;

      case '\n':
	{
	  if ( command != 0 )
	  {
	    ExecuteCommand( command );

	    command = 0;
	  }

	  uart0_printf( "\r\nPGM> " );
	}
	break;

      default:
	{
	  command = curchar;
	  put_char( curchar );
	}
    }
  }
}

//initialize routine
void
Initialize(  )
{
  InitializePLL(  );		// initialize PLL (Jim Lynch function)
  InitializeIO(  );		// set up IO (directions and functions)
  InitializeInterrupts(  );
  uart0_open(  );		// open UART0
  spi1_open(  );		// open SPI1
}

//initialize IO pins
void
InitializeIO(  )
{
  SCS = 0x03;			// select the "fast" version of the I/O ports
  FIO1DIR3 = 0x01;		// set LED bit as output
  FIO1PIN3 = 0x00;		// turn LED on (pin is 0)

  FIO0PIN = 0x00000000;
  PINSEL0 = 0x00000005;		// set up UART0 pins as UART0 instead of GPIO
  PINSEL1 = 0x00000000;		// set up SPI1 pins as GPIO
  FIO0DIR = PIN_TX;		// set RST and TX as outputs (and another control bit on my particular board)
}

void
EnablePROGPin(  )
{
  FIO0PIN &= ~PIN_PROG;		// clear PROG
  FIO0DIR |= PIN_PROG;
}

void
DisablePROGPin(  )
{
  FIO0DIR &= ~PIN_PROG;
}

void
EnableSPIPins(  )
{
  FIO0PIN |= PIN_CSN;		// set CSN
  PINSEL1 = 0x000000A8;		// set up SPI1 pins as SPI1 instead of GPIO
  FIO0DIR &= ~( PIN_MISO );
  FIO0DIR |= PIN_CSN | PIN_MOSI | PIN_SCK;
}

void
DisableSPIPins(  )
{
  PINSEL1 = 0x00000000;
  FIO0DIR &= ~( PIN_CSN | PIN_MOSI | PIN_SCK | PIN_MISO );
}

void
EnableRSTPin(  )
{
  FIO0PIN |= PIN_RST;		// set RST
  FIO0DIR |= PIN_RST;
}

void
DisableRSTPin(  )
{
  FIO0DIR &= ~PIN_RST;
}

void
SetRSTPin(  )
{
  PINS_SET( FIO0PIN, PIN_RST );
}

void
ClearRSTPin(  )
{
  PINS_CLEAR( FIO0PIN, PIN_RST );
}

void
SetCSNPin(  )
{
  PINS_SET( FIO0PIN, PIN_CSN );
}

void
ClearCSNPin(  )
{
  PINS_CLEAR( FIO0PIN, PIN_CSN );
}

void
SetPROGPin(  )
{
  PINS_SET( FIO0PIN, PIN_PROG );
}

void
ClearProgPin(  )
{
  PINS_CLEAR( FIO0PIN, PIN_PROG );
}

//toggles on-board LED
void
ToggleLED(  )
{
  if ( ( FIO1PIN3 & 0x01 ) == 0x00 )
    FIO1PIN3 = 0x01;
  else
    FIO1PIN3 = 0x00;
}

//initialize PLL to 5x (Jim Lynch function)
void
InitializePLL(  )
{
  // Setting Multiplier and Divider values
  PLLCFG = 0x24;
  Feed(  );

  // Enabling the PLL */
  PLLCON = 0x1;
  Feed(  );

  // Wait for the PLL to lock to set frequency
  while ( !( PLLSTAT & PLOCK ) ) ;

  // Connect the PLL as the clock source
  PLLCON = 0x3;
  Feed(  );

  // Enabling MAM and setting number of clocks used for Flash memory fetch (4 cclks in this case)
  MAMCR = 0x2;
  MAMTIM = 0x4;

  VPBDIV = 0x01;
}

//operate PLL Feed (Jim Lynch function)
void
Feed(  )
{
  PLLFEED = 0xAA;
  PLLFEED = 0x55;
}

void
InitializeInterrupts(  )
{
  VICIntEnClr = 0xFFFFFFFF;	// clear all interrupts
  VICIntSelect = 0x00000000;	// clear all FIQ selections

  VICIntSelect &= ~VIC_BIT( VIC_UART0 );	// UART0 selected as IRQ
  VICIntEnable = VIC_BIT( VIC_UART0 );	// UART0 interrupt enabled
  VICVectCntl0 = VIC_ENABLE | VIC_UART0;
  VICVectAddr0 = ( unsigned long ) IRQ_Routine;	// address of the ISR
  VICDefVectAddr = ( unsigned long ) IRQ_Routine;	// point unvectored IRQs to reset()
}

//the following 4 stubs are functions by Jim Lynch
void
IRQ_Routine(  )
{
  while ( ( U0IIR & 0x01 ) == 0 )
  {
    while ( U0LSR & 0x01 )
    {
      tx_buffer[tx_buf_last] = U0RBR;
      tx_buf_last = ( tx_buf_last + 1 ) % BUF_SIZE;
    }
  }

  VICVectAddr = 0x00000000;
}

void
FIQ_Routine(  )
{
  while ( ( U0LSR & 0x20 ) == 0 ) ;

  U0THR = '@';
}

void
SWI_Routine(  )
{
  while ( ( U0LSR & 0x20 ) == 0 ) ;

  U0THR = '#';
}

void
UNDEF_Routine(  )
{
  while ( ( U0LSR & 0x20 ) == 0 ) ;

  U0THR = '$';
}

void
ExecuteCommand( unsigned char command )
{
  switch ( command )
  {
    case 'p':
    case 'P':
      {
	ExecuteCommandProgram(  );
      }
      break;

    case 'd':
    case 'D':
      {
	ExecuteCommandDumpAllMemory(  );
      }
      break;
  }
}

void
ExecuteCommandProgram( void )
{
  unsigned char curchar;
  bool          receivedEOF = false;
  hexRecord     record;
  char          inputRecord[521];
  int           inputRecordIndex = 0;
  unsigned int  i, j, k;

  initialize_flash(  );

  for ( i = 0; i < sizeof( inputRecord ); i++ )
    inputRecord[i] = 0;

  uart0_printf( "\r\n\r\nSend hex file now...\r\n" );

  while ( !receivedEOF )
  {
    curchar = get_char(  );

    if ( curchar == 0 )
      uart0_printf( "Received zero" );

    switch ( curchar )
    {
      case '\r':
	break;

      case '\n':
	ParseHexRecord( &record, inputRecord, inputRecordIndex );

	if ( record.rec_type != HEX_REC_TYPE_EOF )
	  add_hex_record_to_flash( &record );
	else
	  receivedEOF = true;

	break;

      case HEX_REC_START_CODE:
	inputRecordIndex = 0;

	// FALL THROUGH TO PROCESS CHAR INSERTION
      default:
	inputRecord[inputRecordIndex] = curchar;
	inputRecordIndex++;
	break;
    }
  }

  uart0_printf( "\r\nProgramming...\r\n" );

  EnableRSTPin(  );
  EnablePROGPin(  );
  ClearRSTPin(  );
  SetPROGPin(  );

  DelayMS( 10 );
  EnableSPIPins(  );
  SetRSTPin(  );

  DelayUS( 1500 );

  for ( i = 0; i < sizeof( nrf24lu1_flash ); i += NRF24LU1_PAGE_SIZE )
  {
    for ( j = i; j < ( i + NRF24LU1_PAGE_SIZE ); j++ )
    {
      if ( nrf24lu1_flash[j] != 0xFF )
      {
	nrf24lu1_wait_for_flash_ready(  );
	nrf24lu1_flash_cmd_wren(  );
	nrf24lu1_flash_cmd_erase_page( i / NRF24LU1_PAGE_SIZE );

	nrf24lu1_wait_for_flash_ready(  );
	nrf24lu1_flash_cmd_wren(  );
	nrf24lu1_flash_cmd_program( i, &nrf24lu1_flash[i],
	    NRF24LU1_PAGE_SIZE );
	break;
      }
    }
  }

  uart0_printf( "\r\nVerifying...\r\n" );

  for ( i = 0; i < sizeof( nrf24lu1_flash ); i += 32 )
  {
    for ( j = i; j < i + 32; j++ )
    {
      if ( nrf24lu1_flash[j] != 0xFF )
      {
	nrf24lu1_wait_for_flash_ready(  );
	nrf24lu1_flash_cmd_read( i, ( unsigned char * ) &inputRecord, 32 );

	for ( k = 0; k < 32; k++ )
	{
	  if ( inputRecord[k] != nrf24lu1_flash[i + k] )
	  {
	    uart0_printf
		( "Incorrect value at %x: expected %x, but receieved %x\r\n",
		i + k, nrf24lu1_flash[i + k], inputRecord[k] );
	  }
	}

	break;
      }
    }
  }

  DisableSPIPins(  );
  SetPROGPin(  );
  DisablePROGPin(  );
  DisableRSTPin(  );

  uart0_printf( "\r\nProgramming complete...\r\n" );
}

void
ExecuteCommandDumpAllMemory( void )
{
  unsigned int  i, j;
  unsigned char databyte[16];

  uart0_printf( "\r\n" );

  EnableRSTPin(  );
  EnablePROGPin(  );
  ClearRSTPin(  );
  SetPROGPin(  );

  DelayMS( 10 );
  EnableSPIPins(  );
  SetRSTPin(  );

  DelayUS( 1500 );

  nrf24lu1_wait_for_flash_ready(  );

  uart0_printf
      ( "	   00 01 02 03 04 05 06 07 08 09 0a 0b 0c 0d 0e 0f\r\n" );

  for ( i = 0; i < 0x4000; i += 16 )
  {
    if ( i < 0x10 )
    {
      uart0_printf( "000" );
    }
    else if ( i < 0x100 )
    {
      uart0_printf( "00" );
    }
    else if ( i < 0x1000 )
    {
      uart0_printf( "0" );
    }

    uart0_printf( "%x ", i );

    nrf24lu1_flash_cmd_read( ( unsigned short ) i, databyte, 16 );

    for ( j = i; j < ( i + 16 ); j++ )
    {
      if ( databyte[j % 16] < 0x10 )
      {
	uart0_printf( "0" );
      }

      uart0_printf( "%x ", databyte[j % 16] );
    }

    uart0_printf( "\r\n" );
  }

  DisableSPIPins(  );
  SetPROGPin(  );
  DisablePROGPin(  );
  DisableRSTPin(  );
}

void
put_char( char c )
{
  uart0_send( c );
}

char
get_char(  )
{
  char          retchar;

  while ( tx_buf_first == tx_buf_last ) ;

  retchar = tx_buffer[tx_buf_first];
  tx_buf_first = ( tx_buf_first + 1 ) % BUF_SIZE;

  return retchar;
}

void
initialize_flash(  )
{
  int           i;

  for ( i = 0; i < sizeof( nrf24lu1_flash ); i++ )
    nrf24lu1_flash[i] = 0xFF;
}

void
add_hex_record_to_flash( hexRecord * record )
{
  int           i;

  for ( i = 0; i < record->rec_data_len; i++ )
    nrf24lu1_flash[i + record->rec_address] = record->rec_data[i];
}
