#include "nrf24lu1.h"

void
nrf24lu1_flash_cmd_wren( void )
{
  nrf24lu1_execute_command_no_address( NRF24LU1_FLASH_CMD_WREN, NULL, 0,
      false );
}

void
nrf24lu1_flash_cmd_wrdis(  )
{
}

unsigned char
nrf24lu1_flash_cmd_rdsr( void )
{
  unsigned char status;

  nrf24lu1_execute_command_no_address( NRF24LU1_FLASH_CMD_RDSR, &status, 1,
      true );

  return status;
}

void
nrf24lu1_flash_cmd_wrsr(  )
{
}

void
nrf24lu1_flash_cmd_read( unsigned short address, unsigned char *data,
    unsigned int data_len )
{
  nrf24lu1_execute_command_two_byte_address( NRF24LU1_FLASH_CMD_READ, address,
      data, data_len, true );
}

void
nrf24lu1_flash_cmd_program( unsigned short address, unsigned char *data,
    unsigned int data_len )
{
  nrf24lu1_execute_command_two_byte_address( NRF24LU1_FLASH_CMD_PROGRAM,
      address, data, data_len, false );
}

void
nrf24lu1_flash_cmd_erase_page( unsigned char page_num )
{
  nrf24lu1_execute_command_one_byte_address( NRF24LU1_FLASH_CMD_ERASE_PAGE,
      page_num, NULL, 0, false );
}

void
nrf24lu1_flash_cmd_erase_all( void )
{
  nrf24lu1_execute_command_no_address( NRF24LU1_FLASH_CMD_ERASE_ALL, NULL, 0,
      false );
}

void
nrf24lu1_flash_cmd_rdfpcr(  )
{
}

void
nrf24lu1_flash_cmd_rdisip(  )
{
}

void
nrf24lu1_flash_cmd_rdismb(  )
{
}

void
nrf24lu1_flash_cmd_endebug(  )
{
}

void
nrf24lu1_clear_prog(  )
{
  NRF24LU1_PROG_IOREGISTER &= ~NRF24LU1_PROG_PINMASK;
}

void
nrf24lu1_set_prog(  )
{
  NRF24LU1_PROG_IOREGISTER |= NRF24LU1_PROG_PINMASK;
}

bool
nrf24lu1_prog_pin_active(  )
{
  if ( ( NRF24LU1_PROG_IOREGISTER & NRF24LU1_PROG_PINMASK ) != 0 )
    return true;
  else
    return false;
}

void
nrf24lu1_clear_csn(  )
{
  NRF24LU1_CSN_IOREGISTER &= ~NRF24LU1_CSN_PINMASK;
}

void
nrf24lu1_set_csn(  )
{
  NRF24LU1_CSN_IOREGISTER |= NRF24LU1_CSN_PINMASK;
}

bool
nrf24lu1_csn_pin_active(  )
{
  if ( ( NRF24LU1_CSN_IOREGISTER & NRF24LU1_CSN_PINMASK ) != 0 )
    return true;
  else
    return false;
}

void
nrf24lu1_wait_for_flash_ready(  )
{
  while ( nrf24lu1_flash_cmd_rdsr(  ) & NRF24LU1_FSR_RDYN ) ;
}

void
nrf24lu1_execute_command_no_address( unsigned char instruction,
    unsigned char *data, unsigned int data_len, bool copydata )
{
  nrf24lu1_execute_command( instruction, NULL, 0, data, data_len, copydata );
}

void
nrf24lu1_execute_command_one_byte_address( unsigned char instruction,
    unsigned char address, unsigned char *data, unsigned int data_len,
    bool copydata )
{
  nrf24lu1_execute_command( instruction, &address, 1, data, data_len,
      copydata );
}

void
nrf24lu1_execute_command_two_byte_address( unsigned char instruction,
    unsigned short address, unsigned char *data, unsigned int data_len,
    bool copydata )
{
  unsigned char address_arr[2];

  address_arr[0] = ( address >> 8 ) % 256;
  address_arr[1] = address % 256;

  nrf24lu1_execute_command( instruction, address_arr, 2, data, data_len,
      copydata );
}

void
nrf24lu1_execute_command( unsigned char instruction, unsigned char *address,
    unsigned int address_len, unsigned char *data, unsigned int data_len,
    bool copydata )
{
  nrf24lu1_clear_csn(  );

  nrf24lu1_spi_send_read( &instruction, 1, true );
  nrf24lu1_spi_send_read( address, address_len, false );
  nrf24lu1_spi_send_read( data, data_len, copydata );

  nrf24lu1_set_csn(  );
}

void
nrf24lu1_spi_send_read( unsigned char *data, unsigned int len, bool copydata )
{
  unsigned int  count;
  unsigned char tempbyte;

  if ( ( data == NULL ) || len == 0 )
    return;

  for ( count = 0; count < len; count++ )
  {
    if ( copydata != false )
      data[count] = nrf24lu1_spi_send_read_byte( data[count] );
    else
    {
      tempbyte = data[count];
      nrf24lu1_spi_send_read_byte( tempbyte );
    }
  }
}
